/*
	Exemplary solution for CPSPC exercise "The Downtown".
	(C) 2007, Martin Krulis
	
	Source code complies to C99 specification.
*/

#include <stdlib.h>
#include <stdio.h>



#define	RES_OK		0
#define RES_ERROR	1


/*
	Bidirectional linked list nodes and it's operations.
*/
struct list_node {
	int data;
	struct list_node *prev, *next;
};


// Removes a @node from a linked list.
void releaseNode(struct list_node *node) {
	if (!node) return;
	if (node->prev) node->prev->next = node->next;
	if (node->next) node->next->prev = node->prev;
	node->prev = node->next = NULL;
}


// Inserts a @newNode into linked list just after the @listNode.
void insertNode(struct list_node *listNode, struct list_node *newNode) {
	if (!listNode || !newNode) return;
	newNode->next = listNode->next;
	newNode->prev = listNode;
	listNode->next = newNode;
	if (newNode->next) newNode->next->prev = newNode;
}





/*
	FIFO structure (for ints).
	The FIFO has fixed capacity and it's represented by array with cyclic indexes.
*/
struct FIFO {
	int capacity, items, first, last;
	int *data;
};


// Resets the fifo and drops all items in it.
void fifoReset(struct FIFO *fifo) {
	fifo->items = fifo->first = fifo->last = 0;
}


// Prepares a FIFO of given capacity.
void fifoInit(struct FIFO *fifo, int capacity) {
	fifo->capacity = capacity;
	fifo->data = (int*)malloc(capacity * sizeof(int));
	fifoReset(fifo);
}


// Returns true if the fifo is emtpy
int fifoIsEmpty(struct FIFO *fifo) {
	return (fifo->items <= 0);
}


// Pushes a new item x into a fifo.
void fifoPush(struct FIFO *fifo, int x) {
	if (fifo->items >= fifo->capacity) return;
	fifo->data[fifo->last] = x;
	fifo->last = (fifo->last + 1) % fifo->capacity;
	fifo->items++;
}


// Pops item from a fifo a returns it.
int fifoPop(struct FIFO *fifo) {
	if (fifoIsEmpty(fifo)) return 0;
	int res = fifo->data[fifo->first];
	fifo->first = (fifo->first + 1) % fifo->capacity;
	fifo->items--;
	return res;
}





/*
	Graph representation and it's operations.
	Graph is represented by linked list of neighbours stored in array of vertices.
	Note: first list_node item in the list is just a head. We keep vertex outgoing
	degree in this head instead of neighbour index (that is stored in any other
	list_node in the chain).
*/
struct graph {
	int N, M;					// N - number of vertices, M - number of edges.
	struct list_node *vertex;	// array of list_nodes (for each vertex one,
								// indexed from 1 as the vertices are)
};



/*
	Loads a graph from opened file. Graph is loaded as oriented (even though 
	the original graph is not). This way we're creating a random orientation.
*/
int loadGraph(FILE *fp, struct graph *g) {
	if (!fp || !g) return RES_ERROR;
	
	// load number of vertices and edges and allocate the vertex array
	if (fscanf(fp, "%d %d\n", &(g->N), &(g->M)) != 2) return RES_ERROR;
	g->vertex = (struct list_node*)malloc((g->N+1) * sizeof(struct list_node));
	for(int v = 1; v <= g->N; v++) {		// clean up all vertices
		g->vertex[v].data = 0;
		g->vertex[v].next = g->vertex[v].prev = NULL;
	}
	
	// load edges
	int i, j;
	struct list_node *node;
	for(int e = 0; e < g->M; e++) {
		// create a new edge node and load it's data
		node = (struct list_node*)malloc( sizeof(struct list_node) );
		if (fscanf(fp, "%d %d\n", &i, &j) != 2) return RES_ERROR;
		
		// if i has too many outgoing edges and j does not
		if ((g->vertex[i].data >= 3) && (g->vertex[j].data < 3)) {
			// switch edge orientation
			int tmp = i;  i = j;  j = tmp;
		}
		node->data = j;
		insertNode((g->vertex + i), node);
		g->vertex[i].data++;	// increment outgoing degree
	}
	
	return RES_OK;
}


// Stores given grpah into opened file.
int saveGraph(FILE *fp, struct graph *g) {
	if (!fp || !g) return RES_ERROR;
	
	// write down number of vertices and edges
	if (fprintf(fp, "%d %d\n", g->N, g->M) <= 0) return RES_ERROR;
	
	// write down all edges
	struct list_node *p;
	for(int v = 1; v <= g->N; v++) {	// for each vertex...
		p = g->vertex[v].next;
		while(p) {						// for each neighbour...
			if (fprintf(fp, "%d %d\n", v, p->data) <= 0) return RES_ERROR;
			p = p->next;
		}
	}
	
	return RES_OK;
}






/*
	Aditional data for a vertex used in BFS.
*/
struct vertex_data {
	int previous;				// index of previous vertex in BFS
	int generation;				// # of generation of BFS (to check if the data are actual)
	struct list_node *edge;		// pointer to edge that leads from previous vertex to me
};



// Reorient path from vertex v back to first vertex from which the BFS was started.
void reorientPath(struct graph *g, struct vertex_data *vData, int v) {
	g->vertex[v].data++;
	struct list_node *edge;
	while(vData[v].previous != 0) {
		edge = vData[v].edge;
		releaseNode(edge);
		insertNode((g->vertex + v), edge);
		edge->data = v = vData[v].previous;
	}
	g->vertex[v].data--;
}


// Add all neighbours of vertex into fifo and update therir vData information.
void addNeighbours(struct graph *g, struct FIFO *fifo, struct vertex_data *vData, int vertex, int generation) {
	struct list_node *p = g->vertex[vertex].next;
	while(p) {
		if (vData[p->data].generation != generation) {
			fifoPush(fifo, p->data);
			vData[p->data].generation = generation;
			vData[p->data].previous = vertex;
			vData[p->data].edge = p;
		}
		p = p->next;
	}
}



// Main algorithm that fixes orientation on bad vertices.
int fixOrientations(struct graph *g) {
	if (!g) return RES_ERROR;
	
	// allocate and prepare fifo and vertex_data arrays.
	struct FIFO fifo;
	struct vertex_data *vData = (struct vertex_data*)malloc((g->N+1) * sizeof(struct vertex_data));
	fifoInit(&fifo, g->N);
	for(int i = 1; i <= g->N; i++) vData[i].generation = 0;
	
	int generation = 0;
	for(int v = 1; v <= g->N; v++)
		if (g->vertex[v].data > 3) {	// does the vertex need fixing?
			// initialize FIFO and data array
			fifoReset(&fifo);
			fifoPush(&fifo, v);
			vData[v].previous = 0;
			vData[v].generation = ++generation;
			
			// performs BFS till we find a vertex with less-than-three output degree.
			int i;	// actual vertex
			while(!fifoIsEmpty(&fifo) && (g->vertex[v].data > 3)) {
				i = fifoPop(&fifo);
				if (g->vertex[i].data < 3) {	// vertex with "free capacity"
					reorientPath(g, vData, i);
					fifoReset(&fifo);			// reinit. FIFO for another BFS
					fifoPush(&fifo, v);
					vData[v].generation = ++generation;	// start BFS next generation
				} else	// just another vertex
					addNeighbours(g, &fifo, vData, i, generation);
			}
			if (g->vertex[v].data > 3) return RES_ERROR;	// given graph is not planar
		}
		
	return RES_OK;
}




/* Main procedure. */
int main(int argc, char **argv) {	

	// load the graph
	struct graph g;
//	FILE *fp = fopen("town.in", "r");
	FILE *fp = stdin;
	if (loadGraph(fp, &g) != RES_OK) return 1;
//	fclose(fp);
	
	// performs the main algorithm
	int res = fixOrientations(&g);

	// store the graph
//	fp = fopen("town.out", "w");
	fp = stdout;
	if (res == RES_OK) {
		if (saveGraph(fp, &g) != RES_OK) return 2;
	} else
		fprintf(fp, "no");
//	fclose(fp);
	
	return RES_OK;
}
